function m_3_03_registerStemContourToAverageContour
% Project each rotated slice into the reference stlab
%
% inputs:
%   * rotated slice contour coordinates
%   * rotated bundles coordinates
%   * individual slab contour coordinates
%   * reference slab contour coordinates
%
% outputs:
%   * coordinates of transformed slice contour
%   * coordinates of transformed bundles
%
% ------
% Author: David Legland
% e-mail: david.legland@grignon.inra.fr
% Created: 2010-01-26,    using Matlab 7.9.0.529 (R2009b)
% Copyright 2010 INRA - Cepia Software Platform.

% coordinates of individual stem contour
fittedContourDir = fullfile('..', 'slabs', 'tables', 'fitGSC');

% coordinates of slice contour and bundles before transformation
inputContourDir = fullfile('tables', 'contoursRot');
inputBundlesDir = fullfile('tables', 'bundlesRot');

% coordinates of slice contour and bundles after transformation
outputContourDir = fullfile('tables', 'contoursReg');
outputContourDir2 = fullfile('tables', 'contoursRegDp100');
outputBundlesDir = fullfile('tables', 'bundlesReg');

% Create output directories if they do not exist
if ~exist(outputContourDir, 'dir')
    mkdir(outputContourDir);
end
if ~exist(outputContourDir2, 'dir')
    mkdir(outputContourDir2);
end
if ~exist(outputBundlesDir, 'dir')
    mkdir(outputBundlesDir);
end


% Load the reference contour
tab = Table.read(fullfile(fittedContourDir, 'cntCRN200XYFitGSC_mean.txt'));
refContour = [tab.data(1, 1:200)' tab.data(1, 201:end)'];

% extract he list of files to process
list = dir(fullfile(inputContourDir, '*.txt'));

for iSlice = 1:length(list)
    
    name = list(iSlice).name(1:4);
    disp(name);
    
    %% Read the slab contour corresponding to current slice
    
    % read coordinates of the fitted contour of current slice, in
    % millimetres
    stem = name(1:3);
    slab = name(4);
    pattern = 'stem=%s;slab=%s.txt';
    fileName = sprintf(pattern, stem, upper(slab));
    tabSlabContour = Table.read(fullfile(fittedContourDir, 'stem_slab', ...
        fileName));
    slabContour = tabSlabContour.data;
   
    
    %% register the rotated contour of macroscopic slice
    
    % read the (rotated) contour
    tab = Table.read(fullfile(inputContourDir, [name 'CntRot.txt']));
    contour = tab.data;
    
    % allocate memory for results
    res = zeros(size(contour, 1), 8);
    regCnt = zeros(size(contour, 1), 2);
    
    % compute the projection for each contour point
    for i = 1:size(contour, 1)
        % cree un from the center through the current point
        ray = [0 0 contour(i,:)];
        
        % intersection with contour
        inter = intersectRayPolygon(ray, slabContour);
        
        % check result validity
        if size(inter, 1) < 1
            pattern = 'No intersection found for point %d of slice %s';
            msg = sprintf(pattern, i, name);
            disp(msg);
            continue;
        end
        
        % compute current angle
        theta = cart2pol(contour(i,1), contour(i,2));
        
        % convert to degrees, starting from vertical
        thetaDeg = rad2deg(normalizeAngle(theta-pi/2));
        
        % compute distances to origin
        inter = inter(1,:);
        d1 = distancePoints([0 0], contour(i,:));
        d2 = distancePoints([0 0], inter);
        
        % compute relative distance to origin
        dr = d1 / d2;
        
        % calcule intersetion du rayon avec contour moyen
        inter = intersectRayPolygon(ray, refContour);
        
        % distance entre origine et modele
        dm = distancePoints([0 0], inter);
        
        % distance absolue dans le repere du modele
        d3 = dm * dr;
        
        % calcule nouvelles coordonnees cartesiennes
        [x y] = pol2cart(theta, d3);
        
        res(i,:) = [thetaDeg d1 d2 dr dm d3 x y];
        regCnt(i,:) = [x y];
    end
        
    % save registered contour
    tab = Table.create(regCnt, {'x', 'y'});
    fileName = fullfile(outputContourDir, [name 'CntReg.txt']);
    write(tab, fileName, '%-3s %8.5f %8.5f\n');


    %% Save a simplified contour

    % simplify polygonal contour using a tolerance of 100 microns
    regCntDp100 = simplifyPolygon(regCnt, .1);
    
    % save registered contour
    tab = Table.create(regCntDp100, {'x', 'y'});
    fileName = fullfile(outputContourDir2, [name 'CntRegDp100.txt']);
    write(tab, fileName, '%-3s %8.5f %8.5f\n');


    
    %% Register the bundles with the same procedure
    
    % read the (simplified) contour
    tab = Table.read(fullfile(inputBundlesDir, [name 'BunRot.txt']));
    bundles = tab.data;
    
    % allocate memory for results
    res = zeros(size(bundles, 1), 8);
    regBun = zeros(size(bundles, 1), 2);
    
    % compute the projection for each contour point
    for i = 1:size(bundles, 1)
        % cree un from the center through the current point
        ray = [0 0 bundles(i,:)];
        
        % intersection with contour
        inter = intersectRayPolygon(ray, slabContour);
        
        % check result validity
        if size(inter, 1) < 1
            pattern = 'No intersection found for bundle %d of slice %s';
            msg = sprintf(pattern, i, name);
            disp(msg);
            continue;
        end
        
        % compute current angle
        theta = cart2pol(bundles(i,1), bundles(i,2));
        
        % convert to degrees, starting from vertical
        thetaDeg = rad2deg(normalizeAngle(theta-pi/2));
        
        % compute distances to origin
        inter = inter(1,:);
        d1 = distancePoints([0 0], bundles(i,:));
        d2 = distancePoints([0 0], inter);
        
        % compute relative distance to origin
        dr = d1 / d2;
        
        % calcule intersetion du rayon avec contour moyen
        inter = intersectRayPolygon(ray, refContour);
        
        % distance entre origine et modele
        dm = distancePoints([0 0], inter);
        
        % distance absolue dans le repere du modele
        d3 = dm * dr;
        
        % calcule nouvelles coordonnees cartesiennes
        [x y] = pol2cart(theta, d3);
        
        res(i,:) = [thetaDeg d1 d2 dr dm d3 x y];
        regBun(i,:) = [x y];
    end
        
    % save registered contour
    tab = Table.create(regBun, {'x', 'y'});
    fileName = fullfile(outputBundlesDir, [name 'BunReg.txt']);
    write(tab, fileName, '%-3s %8.5f %8.5f\n');

end

