% SEGMENTSLABIMAGES Segment all slab images
%
% Segment all images located in the images/original directory, and save the
% results in the "slab/images/seg" directory.
%

% list of images to process
list = dir('images/original/*.jpg');

for i = 1:length(list)
    % load current image
    name = list(i).name;
    img = imread(fullfile('images', 'original', name));
    
    % use the intensity part, and apply threshold
    hsv = rgb2hsv(img);
    v2 = hsv(:,:,3) > .5;
    clear hsv;
    
    % clean up using morphological closing and opening
    se = strel('disk', 15);
    seg = imclose(imopen(v2, se), se);

    % save global segmentation result
    shortName = name(1:length(name)-4);
    imwrite(seg, fullfile('images', 'seg', [shortName 'Seg.tif']));
    
    
    % convert to label image
    lbl = bwconncomp(seg, 4);
    
    % extract position and extent of each slab
    props = regionprops(lbl, {'centroid', 'boundingbox'});
    
    % compute centroid position and box extent
    ns = length(props);
    xc = zeros(ns, 1);
    yc = zeros(ns, 1);
    box = zeros(ns, 4);
    for s = 1:ns
        xc(s) = props(s).Centroid(1);
        yc(s) = props(s).Centroid(2);
        box(s, :) = props(s).BoundingBox;
    end
    
    % extract "row" and "column" of each slab
    dim = size(img);
    Dy = dim(1);    Dx = dim(2);
    row = floor(3 * yc / Dy) + 1;
    col = floor(3 * xc / Dx) + 1;
    
    stemName = name(1:3);
    recto = name(4) == 'r';
    
    % sort slab images in order: (X->)A->B->...->G(->H)
    [pos inds] = sortrows([row col], [-1 2]);
    
    % determine the digit for the slab index
    if recto
        letters = ['X' char((1:ns-1) + double('a') -1)];
        side = 'r';
    else
        letters = char((1:ns) + double('a') -1);
        side = 'v';
    end
    
    % convert label structure to a matrix
    lbl = labelmatrix(lbl);
    
    % extract each slab sub-image
    for s = 1:ns
        slabName = [stemName letters(s) side];
        disp(['slab: ' slabName]);
        
        % determine the bounds of final image
        rect = floor(box(inds(s), :));
        w = rect(3);
        h = rect(4);
        w1 = floor((600-w)/2);
        w2 = ceil((600-w)/2);
        h1 = floor((600-h)/2);
        h2 = ceil((600-h)/2);
        
        % extract sub-image
        subImg = imcrop(lbl==inds(s), rect+[-w1 -h1 w1+w2-1 h1+h2-1]);
        
        % save the slab subimage uing appropriate name
        imwrite(subImg, fullfile('images', 'segBySlab', [slabName '.tif']));
    end
    
    % remove unused variables for saving memory
    clear lbl;
    clear seg;
    clear res;
    clear v2;
end
