% function m_1_07_fitModelContours(varargin)
%ANOVACP Fit Geno+Stem+Cut linear model to create synthetic contours
%
%
%
% ------
% Author: David Legland
% e-mail: david.legland@grignon.inra.fr
% Created: 2013-05-28,    using Matlab 7.9.0.529 (R2009b)
% Copyright 2013 INRA - Cepia Software Platform.

% read table of selected contours
tab = Table.read(fullfile('tables', 'cntCRN200XY_slab.sel.txt'));

% extract factors
fact = Table.read(fullfile('tables', 'fact2_slab.sel.txt'));
geno = fact('geno');
stem = fact('stem');
cut  = fact('cut');

% create the groups for anova
groups = [geno stem cut];

% number of levels for each factor
nGenos = length(geno.levels{1});
nStems = length(stem.levels{1});
nCuts  = length(cut.levels{1});

% number of factor combinations
nGenoCut = nGenos * nCuts;
nStemCut = nStems * nCuts;

% allocate memory for results
N = 200;
resMean     = zeros(1, 2*N);
resGeno     = zeros(nGenos, 2*N);
resStem     = zeros(nStems, 2*N);
resCut      = zeros(nCuts,  2*N);
resGenoCut  = zeros(nGenoCut, 2*N);
resStemCut  = zeros(nStemCut, 2*N);

% calcule, pour chaque contour a modeliser, les indices du genotype et de
% la position correspondante
indsGeno    = 1:nGenos;
indsStem    = 1:nStems;
indsCut     = [2 1 3:nCuts];

[indsGenoGC indsCutGC] = meshgrid(indsGeno, indsCut);
indsGenoGC  = indsGenoGC(:);
indsCutGC   = indsCutGC(:);

% calcule, pour chaque contour a modeliser, les indices de la tige et de
% la position correspondante
[indsStemSC indsCutSC] = meshgrid(indsStem, indsCut);
indsStemSC  = indsStemSC(:);
indsCutSC   = indsCutSC(:);


%% Linear Model Fitting

% Fit a linear model to each coordinate, and compute corresponding fitted
% values
for i = 1:2*N
    % fit linear model to the current coordinate
    [p table stats] = anova(tab.data(:, i), groups, ...
        'model', 'linear', ...
        'nested', [0 0 0;1 0 0;0 0 0], ...
        'random', 2, ...
        'display', 'off'); %#ok<ASGLU>
   
    % compute the global average
    resMean(i) = stats.coeffs(1);

    % compute average by genotype
    for j = 1:nGenos
        resGeno(j, i) = stats.coeffs(1) + stats.coeffs(j+1);
    end

    % compute average by stem
    for j = 1:nStems
        stemName = stats.grpnames{2}{j};
        indGeno = strcmp('s', stemName) + 1;
        resStem(j, i) = ...
            stats.coeffs(1) + ...               % global average
            stats.coeffs(indGeno+1) + ...       % genotype
            stats.coeffs(j+3);                  % stem
    end
    
    % compute average by cutting position
    for j = 1:nCuts
        resCut(j, i) = ...
            stats.coeffs(1) + ...               % global average
            stats.coeffs(indsCut(j)+19);        % cutting position
    end

    % compute average by combination genotype / cutting position
    for j = 1:nGenoCut
        resGenoCut(j, i) = ...
            stats.coeffs(1) + ...               % global average
            stats.coeffs(indsGenoGC(j)+1) + ... % genotype
            stats.coeffs(indsCutGC(j)+19);      % cutting position
    end

    % compute average by combination stem / cutting position
    for j = 1:nStemCut
        indStem = indsStemSC(j);
        stemName = stats.grpnames{2}{indStem};
        indGeno = strcmp('s', stemName(1)) + 1;
        resStemCut(j, i) = ...
            stats.coeffs(1) + ...               % global average
            stats.coeffs(indGeno+1) + ...       % genotype
            stats.coeffs(indsStemSC(j)+3) + ... % stem
            stats.coeffs(indsCutSC(j)+19);      % cutting position
    end
    
end

%% Create result table objects

% name of columns
colNames = tab.colNames;

% create names for rows
genoNames = strcat('geno=', stats.grpnames{1});
stemNames = char(stats.grpnames{2});
stemNames = strcat('stem=', cellstr(stemNames(:, 1:3)));
cutNames = strcat('cut=', stats.grpnames{3}(indsCut));

tabMean = Table(resMean, colNames, {'mean'}); 
tabGeno = Table(resGeno, colNames, genoNames); 
tabStem = Table(resStem, colNames, stemNames); 
tabCut  = Table(resCut, colNames, cutNames); 

% create result table for geno*cut table
genoCutNames = cell(nGenoCut, 1);
for i = 1:nGenoCut
    genoCutNames{i} = sprintf('geno=%s;cut=%s', ...
        stats.grpnames{1}{indsGenoGC(i)}, ...
        stats.grpnames{3}{indsCutGC(i)});
end
tabGenoCut = Table(resGenoCut, colNames, genoCutNames);

% create result table for stem*cut table
stemCutNames = cell(nStemCut, 1);
for i = 1:nStemCut
    stemCutNames{i} = sprintf('stem=%s;cut=%s', ...
        stats.grpnames{2}{indsStemSC(i)}, ...
        stats.grpnames{3}{indsCutSC(i)});
end
tabStemCut = Table(resStemCut, colNames, stemCutNames);


%% save result files

outputDir = fullfile('tables', 'fitGSC');
if ~exist(outputDir, 'dir')
    mkdir(outputDir);
end
write(tabMean, fullfile(outputDir, 'cntCRN200XYFitGSC_mean.txt'));

write(tabGeno, fullfile(outputDir, 'cntCRN200XYFitGSC_geno.txt'));
write(tabStem, fullfile(outputDir, 'cntCRN200XYFitGSC_stem.txt'));
write(tabCut,  fullfile(outputDir, 'cntCRN200XYFitGSC_cut.txt'));

write(tabGenoCut, fullfile(outputDir, 'cntCRN200XYFitGSC_geno_cut.txt'));
write(tabStemCut, fullfile(outputDir, 'cntCRN200XYFitGSC_stem_cut.txt'));


%% Save result polygons

polyMean = rowToPolygon(resMean, 'packed');
tab = Table(polyMean, {'x', 'y'});
if ~exist(fullfile(outputDir, 'mean'), 'dir')
    mkdir(fullfile(outputDir, 'mean'));
end
write(tab, fullfile(outputDir, 'mean', 'cntCRN200FitGSC_mean.txt'));
