function m_1_08_computeContours_stem_slab(varargin)
%MEANCONTOUR_STEM_SLAB  Calcule un contour moyen par couple bloc-tige
%
%   Pour chaque bloc, on considere les deux coupes avant et apres, 
%   et on calcule la moyenne des coordonnes de chaque point du contour.
%
%   Usage :
%   output = meanContour_stem_slab(input)
%
%   Example
%   meanContour_stem_slab
%
%   See also
%
%
% ------
% Author: David Legland
% e-mail: david.legland@grignon.inra.fr
% Created: 2009-10-14,    using Matlab 7.8.0.347 (R2009a)
% Copyright 2009 INRA - Cepia Software Platform.


%% Initialisations

% directory containing fitted contours
repTbl      = fullfile('tables', 'fitGSC');

% name of the different stems
stemNames   = {...
    'm01', 'm02', 'm03', 'm04', 'm09', 'm11', 'm12', 'm14', ...
    's02', 's03', 's05', 's07', 's08', 's09', 's11', 's12'};
nStems  = length(stemNames);

% name of the different cutting positions
cutNames    = {'xa', 'ab','bc', 'cd', 'de', 'ef', 'fg', 'gh'};

% name of the slabs, located between two cuts
slabNames   = {'A', 'B', 'C', 'D', 'E', 'F', 'G'};
nSlabs  = length(slabNames);

% number of slab contours to compute 
nModels = nStems * nSlabs;

% read coordinates of all fitted contours
tab = Table.read(fullfile(repTbl, 'cntCRN200XYFitGSC_stem_cut.txt'));
nCoords = size(tab, 2);


%% Compute interpolated contours

% allocate new data table
res = Table(zeros(nModels, nCoords));

% iteration for the different stem
for iStem = 1:nStems
    
    stem = stemNames{iStem};
   
    % iterations for the different slabs
    for iSlab = 1:nSlabs
        % current slab
        slab    = slabNames{iSlab};
        
        % the two cut positions (before and after)
        cut1    = cutNames{iSlab};
        cut2    = cutNames{iSlab+1};
        
        % le noms des lignes correspondantes dans le fichier de donnees
        pattern = 'stem=%s(%s);cut=%s';
        name1   = sprintf(pattern, stem, stem(1), cut1);
        name2   = sprintf(pattern, stem, stem(1), cut2);
        
        % extrait les lignes
        row1    = tab(name1, :).data;
        row2    = tab(name2, :).data;
        
        % calcule le contour moyen pour la combinaison stem/slab
        rowSlab = (row1 + row2 ) / 2;
        
        % remplit la structure de donnees de resultats
        ind = (iStem - 1) * nSlabs + iSlab;
        res.rowNames{ind} = sprintf('stem=%s;slab=%s', stem, slab);
        res.data(ind, :) = rowSlab;
    end
end


%% Sauvegarde fichier

% garde les memes noms de variable
res.colNames = tab.colNames;
write(res, fullfile(repTbl, 'cntCRN200XYFitGSC_stem_slab.txt'));


%% Sauvegarde des fichiers par contour de bloc

% verifie existence du repertoire de sortie
repRes = fullfile(repTbl, 'stem_slab');
if ~exist(repRes, 'dir')
    mkdir(repRes);
end

% sauve un fichier pour chaque contour stem/slab
nPoints = 200;
varNames = {'x', 'y'};
rowNames = num2str((1:nPoints)', '%d');
for i = 1:nModels
    row = res(i, :).data;
    name = res.rowNames{i};
    
    % convert row to polygon
    poly = [row(1:nPoints)' row(nPoints+1:end)'];
    
    % create result table
    tab = Table(poly, varNames, rowNames);
    
    write(tab, fullfile(repRes, [name '.txt']));
end



